// cl /LD /MD /GX THE_TOWER.cpp
// と打ってさくっとこんぱいるっ!

#include "Saori.cpp" //!


// babel用マクロ定義
#define __UNICODE_CHAR_SIZE_2__
#define __LITTLE_ENDIAN_COMPUTER__
#define __USING_SJIS__
#define __BBL_USING_STATIC_TABLE__
#define __BBL_STRICT_TRANSLATE__

#include "babel/babel.cpp"


using namespace std;


// fools' broken tower クラス
class THE_TOWER : public saori_interface<THE_TOWER, lazylodable<THE_TOWER> >
{
public:

    // コンストラクタで追加する方法は美しくないと思った。まあいいや。
    THE_TOWER()
    {
        set_myname("THE TOWER");
        add_slot("analyze encoding", analyze_encoding);
        add_slot("auto translate", auto_translate);
        add_slot("semiauto translate", semiauto_translate);
        add_slot("translate", translate);
        // babelの初期化
        // ここではしたくないけど他にするところがない。
        babel::init_babel();
    }
    

private:
    
    int get_encoding(const std::string &str, const std::string &func_name)
    {
        using namespace babel::base_encoding;
        
        if(str == "Shift_JIS")
            return sjis;
        else if(str == "EUC-JP")
            return  euc;
        else if(str == "UTF-8")
            return utf8;
        else if(str == "ISO-2022-JP")
            return iso2022jp;
        else if(str == "JIS")
            return jis;
        else
            throw saori_error(SAORI_400_Bad_Request, "'THE TOWER::" + func_name + "' Unknown encoding name.");
    }
    
protected:

    // エンコードの解析
    volatile bool analyze_encoding(const ArgumentType& args, ResultPair &res)
    {
        using namespace babel::base_encoding;
        
        if(args.size() < 2)
            throw saori_error(SAORI_400_Bad_Request, "'THE TOWER::analyze encoding' Argument error.");
        
        switch (babel::analyze_base_encoding(args[1]))
        {
        case ansi:
            res.first = "ANSI";
            break;
        case sjis:
            res.first = "Shift_JIS";
            break;
        case euc:
            res.first = "EUC-JP";
            break;
        case utf8:
            res.first = "UTF-8"; 
            break;
        case iso2022jp:
            res.first = "ISO-2022-JP";
            break;
        case jis:
            res.first = "JIS";
            break;
        default:
            res.first = "unknown";
        }
        
        set_charset("UTF-8");
        
        return true;
    }
    
    
    // 自動解析変換
    volatile bool auto_translate(const ArgumentType& args, ResultPair &res)
    {
        if(args.size() < 2)
            throw saori_error(SAORI_400_Bad_Request, "'THE TOWER::auto translate' Argument error.");

        // 変換先のエンコードが指定されていないときは
        // OSのエンコード(Shift-JIS)になる。
        if(args.size() == 2 || args[2] == ""){
            res.first = babel::auto_translate<babel::bbl_default_string>(args[1]);
            res.second.push_back("Shift_JIS");
            set_charset("Shift_JIS");
        }else{
            res.first = babel::auto_translate<babel::bbl_default_string>(args[1], get_encoding(args[2], "auto translate"));
            res.second.push_back(args[2]);
            set_charset(args[2]);
        }
        return true;
    }
    
    
    // Charsetを見て変換する。
    volatile bool semiauto_translate(const ArgumentType& args, ResultPair &res)
    {
        if(args.size() < 2)
            throw saori_error(SAORI_400_Bad_Request, "'THE TOWER::semiauto translate' Argument error.");

        // 変換先のエンコードが指定されていないときは
        // OSのエンコード(Shift-JIS)になる。
        if(args.size() == 2 || args[2] == ""){
            res.first = babel::translate_to_string<babel::bbl_default_string>(
                            args[1], 
                            get_encoding(get_charset(), "semiauto translate"));
            res.second.push_back("Shift_JIS");
            set_charset("Shift_JIS");
        }else{
            res.first = babel::translate_to_string<babel::bbl_default_string>(
                            args[1], 
                            get_encoding(get_charset(), "semiauto translate"),
                            get_encoding(args[2], "semiauto translate"));
            res.second.push_back(args[2]);
            set_charset(args[2]);
        }
        return true;
    }
    
    
    // 変換元、変換先両方とも指定する。
    // Charsetヘッダの指定は無視。
    volatile bool translate(const ArgumentType& args, ResultPair &res)
    {
        if(args.size() < 3)
            throw saori_error(SAORI_400_Bad_Request, "'THE TOWER::translate' Argument error.");

        // 変換先のエンコードが指定されていないときは
        // OSのエンコード(Shift-JIS)になる。
        if(args.size() == 3 || args[3] == ""){
            res.first = babel::translate_to_string<babel::bbl_default_string>(
                            args[1], 
                            get_encoding(args[2], "translate"));
            res.second.push_back("Shift_JIS");
            set_charset("Shift_JIS");
        }else{
            res.first = babel::translate_to_string<babel::bbl_default_string>(
                            args[1], 
                            get_encoding(args[2], "translate"),
                            get_encoding(args[3], "translate"));
            res.second.push_back(args[3]);
            set_charset(args[3]);
        }
        return true;
    }
    
};


// おまじない
SAORI_INTERFACE_MODULE(THE_TOWER)

